# -*- coding: utf-8 -*-
import os

from flask import Flask

import settings


def _config_app(app):
    """set flask config
    """
    app.config.from_object(settings.SYSTEM_CONFIG_PATH)


def _config_ext(app):
    """set config of extensions
    """
    pass


def _set_blueprints(app):
    for bp, prefix in settings.SYSTEM_BLUEPRINTS:
        print "blueprint: %s, prefix: %s" % (bp.name, prefix)
        print "root_path: " + bp.root_path
        app.register_blueprint(bp, url_prefix=prefix)


def create_app(app_name):
    """create Flask app
    """
    app = Flask(app_name)
    _config_app(app)
    _config_ext(app)
    _set_blueprints(app)

    return app



# -*- coding: utf-8 -*-


class BaseCommand(object): 
    def __init__(self):
        pass

    def execute(self, *args):
        pass

    def add_arguments(self, subcmd):
        """parse subcmd from list to directory with key you set
        """        
        pass


# -*- coding: utf-8 -*-

import os

# blueprint
#from apps.myapp.views import myapp

# config
SYSTEM_CONFIG_NAME = "%(project_name)s".upper() + "_CONFIG"
SYSTEM_CONFIG_PATH = "%(project_name)s.configs.base.BaseConfig"
env_config = os.environ.get(SYSTEM_CONFIG_NAME)
if env_config:
    SYSTEM_CONFIG_PATH = "%(project_name)s.configs." + env_config


# blueprint
SYSTEM_BLUEPRINTS = (
    #(myapp, None),
)



# -*- coding: utf-8 -*-

from flask.ext.script import Manager

from %(project_name)s import create_app


app = create_app(app_name="%(project_name)s")
mgr = Manager(app)


@mgr.command
def run():
    app.run()


if __name__ == "__main__":
    mgr.run()



# -*- coding: utf-8 -*- 

import sys
from importlib import import_module

app_name = "flask_go"
commands = {
    "startproject": "create a project",
    "startapp": "create a blueprint app",
}


def load_command_class(name):
    module = import_module("%s.commands.%s" % (app_name, name))
    return module.Command()


def get_command(name):
    try:
        commands[name]
    except:
        print "Unknow command: %r" % name
        return None

    return load_command_class(name)


def help_text():
    text = [
        "Available subcommands:",
        ""
    ]

    for key, value in commands.iteritems():
        cmd_text = "%s\t\t%s" % (key, value)
        text.append(cmd_text)

    return "\n".join(text) + "\n\n"


def execute_from_command_line():
    try:
        cmd = sys.argv[1]
    except IndexError:
        cmd = "help"

    try:
        subcmd = sys.argv[2:]
    except:
        print "error"

    if cmd == "help":
        sys.stdout.write(help_text())
    else:
        exe = get_command(cmd)
        if exe:
            exe.execute(subcmd)



# -*- coding: utf-8 -*- 

from setuptools import setup, find_packages


EXCLUDE_FROM_PACKAGES = [
    'flask_go.project_template',
    #'flask_go.project_template.project_name',
    'flask_go.app_template'
]

setup(
    name='flask-go',
    version='0.1.1',
    description='Let you create flask project like use django-admin',
    url='https://github.com/CelloCello/flask-go',
    author='Cello Hsueh',
    author_email="cello1124@gmail.com",
    license='BSD',
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data=True,
    scripts=['bin/flask-go.py'],
    entry_points={'console_scripts': [
        'flask-go = flask_go.management:execute_from_command_line',
    ]},
    install_requires=[
        'flask',
        'flask-script',
    ], 
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]    
)



# -*- coding: utf-8 -*-
import os
import argparse
import re

import flask_go
from base import BaseCommand


class Command(BaseCommand): 
    def __init__(self):
        pass

    def execute(self, subcmd):
        subcmd_dict = self.parse_subcmd(subcmd)
        app_name = subcmd_dict["app_name"]
        directory = subcmd_dict["directory"]
        tmp_from_path = subcmd_dict["template"]
        if directory:
            directory = os.path.abspath(os.path.expanduser(directory))
            directory = os.path.join(directory, app_name)
        else:
            directory = os.path.join(os.getcwd(), app_name)

        template_path = self.get_template(tmp_from_path)
        prefix_length = len(template_path) + 1
        #print "temp path: " + template_path
        print "create projcet [%s] in direct: %s" % (app_name, directory)
        for root, dirs, files in os.walk(template_path):
            # make dirs
            path_rest = root[prefix_length:]
            root_reset = path_rest.replace("app_template", app_name)
            root_reset = root_reset.replace("app_name", app_name)
            dir_path = os.path.join(directory, root_reset)
            if not os.path.exists(dir_path):
                os.mkdir(dir_path)

            # read template files and write to directory
            for filename in files:
                if filename.endswith(('.pyo', '.pyc', '.py.class')):
                    # Ignore some files as they cause various breakages.
                    continue

                # read template
                tmp_path = os.path.join(root, filename)
                with open(tmp_path, 'rb') as tmp_file:
                    content = tmp_file.read()
                    content = re.sub(r"%(?!\(\w+\)s)", "%%", content)
                    content %= subcmd_dict

                # write to directory
                dest_path = os.path.join(dir_path, filename.replace("app_name", app_name))
                with open(dest_path, 'wb') as new_file:
                    new_file.write(content)

    def parse_subcmd(self, subcmd):
        """parse subcmd from list to directory with key you set
        """
        parser = argparse.ArgumentParser()
        parser.add_argument('app_name', help='Name of the application or project.')
        parser.add_argument('directory', nargs='?', help='Optional destination directory')
        parser.add_argument('--template', help='The path to load the template from.')
        args = parser.parse_args(subcmd)
        return vars(args)

    def get_template(self, from_path):
        if from_path is None:
            return os.path.join(flask_go.__path__[0], "app_template")
        else:
            expanded_template = os.path.expanduser(from_path)
            expanded_template = os.path.normpath(expanded_template)
            if os.path.isdir(expanded_template):
                return expanded_template

        raise Exception("error in template: " + from_path)        